@@ -42,8 +42,8 @@
 #define NEW_ROW_SEPARATOR(box) \
 	gtk_box_pack_start(GTK_BOX(box), gtk_hseparator_new(), FALSE, FALSE, 0);
 
-static guchar *grg_pref_file = NULL;
-static guchar *grg_prefs_editor_font = NULL;
+static gchar *grg_pref_file = NULL;
+static gchar *grg_prefs_editor_font = NULL;
 
 static guchar tmp_pref_crypto;
 static guchar tmp_pref_hash;
@@ -54,7 +54,7 @@ static GtkWidget *file_entry;
 
 static gboolean active_flag = FALSE;
 
-//radio buttons & other things to update
+/*radio buttons & other things to update */
 static GtkWidget *rij1_but, *ser_but, *twof_but, *cast_but, *safer_but;
 static GtkWidget *rij2_but, *tdes_but, *loki_but, *sha_but, *ripe_but;
 static GtkWidget *zlib_but, *bz_but, *r0_but, *r3_but, *r6_but, *r9_but;
@@ -68,10 +68,11 @@ gboolean grg_prefs_bak_files = TRUE;
 gboolean grg_prefs_splash = TRUE;
 gboolean grg_prefs_clip_clear_on_close = FALSE;
 gboolean grg_prefs_clip_clear_on_quit = TRUE;
-gint grg_prefs_xpire = EXP_TIME_DEF;	//abs(x)= num of days; < 0 = never
+gint grg_prefs_xpire = EXP_TIME_DEF;	/*abs(x)= num of days; < 0 = never */
 gint grg_prefs_wipe_passes = WIPE_PASSES_DEF;
+gint grg_prefs_mainwin_width = -1, grg_prefs_mainwin_height = -1;
 
-guchar *
+gchar *
 get_pref_file (void)
 {
 	if (grg_pref_file)
@@ -80,13 +81,13 @@ get_pref_file (void)
 }
 
 void
-set_pref_file (const guchar * newval)
+set_pref_file (const gchar * newval)
 {
 	g_free (grg_pref_file);
 	grg_pref_file = g_strdup (newval);
 }
 
-guchar *
+gchar *
 get_pref_font_string (void)
 {
 	if (grg_prefs_editor_font)
@@ -95,7 +96,7 @@ get_pref_font_string (void)
 }
 
 void
-set_pref_font_string (const guchar * newval)
+set_pref_font_string (const gchar * newval)
 {
 	g_free (grg_prefs_editor_font);
 	grg_prefs_editor_font = g_strdup (newval);
@@ -104,7 +105,7 @@ set_pref_font_string (const guchar * newval)
 void
 set_pref_font_string_from_editor (void)
 {
-	guchar *newval = get_editor_font ();
+	gchar *newval = get_editor_font ();
 	set_pref_font_string (newval);
 	g_free (newval);
 }
@@ -144,7 +145,7 @@ reset_values (GtkWidget * parent)
 {
 	if (grg_load_prefs () != GRG_OK)
 	{
-		guchar *msg =
+		gchar *msg =
 			_("Invalid preferences file. Resetting to defaults.");
 		grg_msg (msg, GTK_MESSAGE_WARNING, parent);
 		grg_prefs_reset_defaults ();
@@ -326,9 +327,14 @@ modify_passes (GtkWidget * data, gpointer value)
 static void
 modify_font (GtkWidget * data, gpointer value)
 {
-	GtkWidget *font_selector, *wait;
+	GtkWidget *font_selector;
+#if 0
+    GtkWidget *wait;
+#endif
 	gint response;
+#if 0
 	gchar *selection = NULL;
+#endif
 	PangoFontDescription *pfd;
 
 	font_selector = gtk_font_selection_dialog_new ("Select a font...");
@@ -355,26 +361,33 @@ modify_font (GtkWidget * data, gpointer value)
 static void
 meta_open_startup_file (GtkWidget * wid, gpointer value)
 {
-	GtkWidget *file_selector;
+    GtkWidget *file_chooser;
 	gint response;
-	file_selector = gtk_file_selection_new (_("Select file..."));
-	gtk_window_set_transient_for (GTK_WINDOW (file_selector),
-				      GTK_WINDOW (value));
-	gtk_widget_show (file_selector);
-	response = gtk_dialog_run (GTK_DIALOG (file_selector));
 
-	if (response == GTK_RESPONSE_OK)
+    file_chooser = gtk_file_chooser_dialog_new (_("Select file..."),
+            GTK_WINDOW(value),
+            GTK_FILE_CHOOSER_ACTION_OPEN,
+            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+            GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+            NULL);
+
+	response = gtk_dialog_run (GTK_DIALOG (file_chooser));
+
+    if (response == GTK_RESPONSE_ACCEPT)
 	{
+        gchar * filename;
+        filename = gtk_file_chooser_get_filename (
+                GTK_FILE_CHOOSER (file_chooser)
+                );
+
 		gchar *utf =
-			g_filename_to_utf8 (gtk_file_selection_get_filename
-					    (GTK_FILE_SELECTION
-					     (file_selector)), -1, NULL, NULL,
-					    NULL);
+			g_filename_to_utf8 (filename, -1, NULL, NULL, NULL);
+        g_free (filename);
 		gtk_entry_set_text (GTK_ENTRY (file_entry), utf);
 		g_free (utf);
 	}
 
-	gtk_widget_destroy (file_selector);
+    gtk_widget_destroy (file_chooser);
 }
 
 static void
@@ -412,7 +425,7 @@ grg_pref_dialog (GtkWidget * parent)
 					     GTK_STOCK_CANCEL,
 					     GTK_RESPONSE_CANCEL, NULL);
 
-	//first page: algorithms
+	/*first page: algorithms */
 	tab1 = gtk_table_new (3, 2, FALSE);
 
 	frame1 = gtk_frame_new (_("Encryption"));
@@ -509,7 +522,7 @@ grg_pref_dialog (GtkWidget * parent)
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (prefs)->vbox), notebook,
 			    TRUE, TRUE, GRG_PAD);
 
-	//second page: General options
+	/*second page: General options */
 	tab2 = gtk_vbox_new (FALSE, GRG_PAD);
 	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), tab2,
 				  gtk_label_new (_("General options")));
@@ -524,7 +537,7 @@ grg_pref_dialog (GtkWidget * parent)
 
 	fdesc = pango_font_description_from_string (grg_prefs_editor_font);
 	gtk_widget_modify_font (but_font, fdesc);
-	g_free (fdesc);
+	pango_font_description_free (fdesc);
 
 	g_signal_connect (G_OBJECT (but_font), "clicked",
 			  G_CALLBACK (modify_font), prefs);
@@ -579,7 +592,7 @@ grg_pref_dialog (GtkWidget * parent)
 			  G_CALLBACK (modify_over), NULL);
 	gtk_box_pack_start (GTK_BOX (box_save), over_check, FALSE, TRUE, 1);
 
-	//third page: Security
+	/*third page: Security */
 	tab3 = gtk_vbox_new (FALSE, GRG_PAD);
 	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), tab3,
 				  gtk_label_new (_("Security")));
@@ -606,7 +619,7 @@ grg_pref_dialog (GtkWidget * parent)
 	gtk_box_pack_start (GTK_BOX (box_xpire), xpire_spin, FALSE, TRUE, 1);
 	gtk_box_pack_start (GTK_BOX (box_xpire), xpire_lbl, FALSE, TRUE, 1);
 
-	//this means "passes in wiping a file", not "wipe the passes" :)
+	/*this means "passes in wiping a file", not "wipe the passes" :) */
 	frame_passes = gtk_frame_new (_("Wipe passes"));
 	gtk_box_pack_start (GTK_BOX (tab3), frame_passes, FALSE, TRUE, 1);
 
@@ -646,7 +659,7 @@ grg_pref_dialog (GtkWidget * parent)
 			  G_CALLBACK (modify_qclip), NULL);
 	gtk_box_pack_start (GTK_BOX (box_clip), qclip_check, FALSE, TRUE, 1);
 
-	//end of last tab
+	/*end of last tab */
 	active_flag = TRUE;
 	reset_values (prefs);
 	update_buttons ();
