@@ -70,36 +70,47 @@
 static void
 meta_browse (GtkWidget * data, GtkWidget * entry)
 {
-	GtkWidget *file_selector;
+    GtkWidget *file_chooser;
+	gint response;
+
 	GtkWidget *dlg =
 		GTK_WIDGET (GTK_WIDGET ((GTK_WIDGET (data->parent))->parent)->
 			    parent);
 	gint res;
 
-	file_selector = gtk_file_selection_new (_("Open..."));
-	gtk_window_set_transient_for (GTK_WINDOW (file_selector),
-				      GTK_WINDOW (dlg));
-	gtk_widget_show (file_selector);
-	res = gtk_dialog_run (GTK_DIALOG (file_selector));
-	if (res == GTK_RESPONSE_OK)
+    file_chooser = gtk_file_chooser_dialog_new (_("Open..."),
+            GTK_WINDOW (dlg),
+            GTK_FILE_CHOOSER_ACTION_OPEN,
+            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+            GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+            NULL);
+
+	response = gtk_dialog_run (GTK_DIALOG (file_chooser));
+    if (response == GTK_RESPONSE_ACCEPT)
 	{
+        gchar *filename;
+
+        filename = gtk_file_chooser_get_filename (
+                GTK_FILE_CHOOSER (file_chooser)
+                );
+
 		gchar *ufile =
-			g_filename_to_utf8 (gtk_file_selection_get_filename
-					    (GTK_FILE_SELECTION
-					     (file_selector)), -1, NULL, NULL,
+			g_filename_to_utf8 (filename, -1, NULL, NULL,
 					    NULL);
+        g_free(filename);
 		gtk_entry_set_text (GTK_ENTRY (entry), ufile);
 		g_free (ufile);
 	}
-	gtk_widget_destroy (file_selector);
+    gtk_widget_destroy (file_chooser);
 }
 
 static GRG_KEY
 read_pwd_file (const gchar * path, GtkWidget * dlg, gboolean X)
 {
-	GtkWidget *wait;
+	GtkWidget *wait = NULL;
 	gint fd, len;
-	gchar *pwd, *upath;
+	gchar *upath;
+    guchar *pwd;
 	GRG_KEY key;
 	struct stat buf;
 
@@ -140,9 +151,9 @@ read_pwd_file (const gchar * path, GtkWidget * dlg, gboolean X)
 static GRG_KEY
 read_pwd_disk (GtkWidget * dlg, gboolean X)
 {
-	GtkWidget *wait;
+	GtkWidget *wait = NULL;
 	gint fd, len;
-	gchar *file;
+	guchar *file;
 	GRG_KEY key = NULL;
 
 #if defined(BLOCK_DEV_IS_FLOPPY) && defined(HAVE_LINUX_FD_H)
@@ -151,7 +162,7 @@ read_pwd_disk (GtkWidget * dlg, gboolean X)
 
 	fd = open (BLOCK_DEV, O_RDONLY);
 
-	//check 4 disk presence
+	/*check 4 disk presence*/
 	if (fd < 0)
 	{
 		report_err (_
@@ -161,7 +172,7 @@ read_pwd_disk (GtkWidget * dlg, gboolean X)
 	}
 
 #if defined(BLOCK_DEV_IS_FLOPPY) && defined(HAVE_LINUX_FD_H)
-	//check for write protection
+	/*check for write protection*/
 	ioctl (fd, FDGETDRVSTAT, &fstruct);
 
 	if ((fstruct.flags >> FD_DISK_WRITABLE_BIT) & 1)
@@ -242,8 +253,8 @@ vis_quality (gpointer ignore, gpointer type)
 	{
 	case TYPE_PWD:
 	{
-		gint bout;
-		gchar *sq;
+		gsize bout;
+		gchar *sq = NULL;
 
 		if (!mapIsUTF)
 			sq = g_locale_from_utf8 (gtk_entry_get_text
@@ -251,12 +262,12 @@ vis_quality (gpointer ignore, gpointer type)
 						 NULL, &bout, NULL);
 
 		gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (quality),
-					       grg_ascii_pwd_quality (mapIsUTF
+					       grg_ascii_pwd_quality ((guchar*)(mapIsUTF
 								      ?
 								      gtk_entry_get_text
 								      (GTK_ENTRY
 								       (question))
-								      : sq,
+								      : sq),
 								      g_utf8_strlen
 								      (gtk_entry_get_text
 								       (GTK_ENTRY
@@ -270,12 +281,12 @@ vis_quality (gpointer ignore, gpointer type)
 	case TYPE_FILE:
 	{
 		gchar *upath =
-			g_filename_from_utf8 ((guchar *)
+			g_filename_from_utf8 (
 					      gtk_entry_get_text (GTK_ENTRY
 								  (file_entry)),
 					      -1, NULL, NULL, NULL);
 		gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (quality),
-					       grg_file_pwd_quality (upath));
+					       grg_file_pwd_quality ((guchar*)upath));
 		g_free (upath);
 	}
 		break;
@@ -385,10 +396,10 @@ grg_new_pwd_dialog (GtkWidget * parent)
 	gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (dialog)->vbox),
 				     quality);
 
-	//show all...
+	/*show all...*/
 	gtk_widget_grab_focus (question);
 	gtk_widget_show_all (GTK_DIALOG (dialog)->vbox);
-	//...but the file selection part
+	/*...but the file selection part*/
 	gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (dialog)->vbox),
 				     hbox_file);
 	gtk_box_reorder_child (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox_file,
@@ -440,14 +451,14 @@ grg_new_pwd_dialog (GtkWidget * parent)
 				break;
 			}
 
-			key = grg_key_gen (ret1, pwd_len);
+			key = grg_key_gen ((guchar*)ret1, pwd_len);
 
 			exit = TRUE;
 			break;
 		}
 		case TYPE_FILE:
 		{
-			const guchar *path =
+			const gchar *path =
 				gtk_entry_get_text (GTK_ENTRY (file_entry));
 
 			key = read_pwd_file (path, dialog, TRUE);
@@ -615,13 +626,13 @@ grg_ask_pwd_dialog (GtkWidget * parent)
 		switch (curr_type_pwd_req)
 		{
 		case TYPE_PWD:
-			key = grg_key_gen (gtk_entry_get_text
+			key = grg_key_gen ((guchar*)gtk_entry_get_text
 					   (GTK_ENTRY (entry)), -1);
 			exit = TRUE;
 			break;
 		case TYPE_FILE:
 		{
-			const guchar *path =
+			const gchar *path =
 				gtk_entry_get_text (GTK_ENTRY (entry));
 			key = read_pwd_file (path, dlg, TRUE);
 
@@ -715,15 +726,15 @@ grg_get_cmdline_key (void)
 		if (!mapIsUTF)
 		{
 			gchar *UTF8d;
-			gint ulen;
+			gsize ulen;
 
 			UTF8d = g_locale_to_utf8 (pwd, -1, NULL, &ulen, NULL);
-			ret = grg_key_gen (UTF8d, ulen);
+			ret = grg_key_gen ((guchar*)UTF8d, ulen);
 			GRGFREE (UTF8d, ulen);
 			ulen = 0;
 		}
 		else
-			ret = grg_key_gen (pwd, -1);
+			ret = grg_key_gen ((guchar*)pwd, -1);
 		GRGAFREE (pwd);
 	}
 		break;
@@ -733,7 +744,7 @@ grg_get_cmdline_key (void)
 		if (!mapIsUTF)
 		{
 			gchar *UTF8d;
-			gint ulen;
+			gsize ulen;
 
 			UTF8d = g_locale_to_utf8 (pwd, -1, NULL, &ulen, NULL);
 			ret = read_pwd_file (UTF8d, NULL, FALSE);
