@@ -20,6 +20,7 @@
  */
 #include <string.h>
 #include <stdio.h>
+#include <stdlib.h>
 
 #include <gtk/gtk.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
@@ -27,6 +28,8 @@
 #include "grg_defs.h"
 #include "grg_pix.h"
 #include "../pixmaps/gringotts.xpm"
+#include "gringotts.h"
+#include "grg_widgets.h"
 
 gboolean mapIsUTF = FALSE;
 
@@ -62,7 +65,7 @@ report_err (gchar * msg, gboolean X, gboolean doquit, GtkWidget * parent)
 	{
 		fprintf (stderr, "%s: %s\n", _("Error"), msg);
 		if (doquit)
-			emergency_quit (); //should suffice quit(1), but just in case... ;-)
+			emergency_quit (); /*should suffice quit(1), but just in case... ;-) */
 	}
 }
 
@@ -217,7 +220,7 @@ grg_window_set_icon (GtkWindow * w)
 	GdkPixbuf *gp = gdk_pixbuf_new_from_xpm_data (gringotts_xpm);
 	GList *gl = NULL;
 
-	g_list_append (gl, gp);
+	gl = g_list_append (gl, gp);
 	gtk_window_set_icon (w, gp);
 	gtk_window_set_default_icon_list (gl);
 
@@ -229,7 +232,7 @@ grg_window_set_icon (GtkWindow * w)
  * grg_find_dialog:
  */
 gboolean
-grg_find_dialog (guchar ** needle, gboolean * only_current,
+grg_find_dialog (gchar ** needle, gboolean * only_current,
 		 gboolean * case_sens, GtkWindow * parent)
 {
 	GtkWidget *dialog, *question, *label, *chk1, *chk2;
@@ -297,8 +300,9 @@ grg_find_dialog (guchar ** needle, gboolean * only_current,
 }
 
 static gboolean
-exit_wait_timed (GtkWidget * w)
+exit_wait_timed (gpointer void_w)
 {
+    GtkWidget * w = (GtkWidget *)void_w;
 	gtk_dialog_response (GTK_DIALOG (w), GTK_RESPONSE_DELETE_EVENT);
 	return FALSE;
 }
@@ -336,7 +340,7 @@ grg_wait_msg (gchar * reason, GtkWidget * parent)
 
 	gtk_widget_show_all (wait);
 
-	gtk_timeout_add (GRG_VISUAL_LATENCY, (GtkFunction) exit_wait_timed,
+	g_timeout_add (GRG_VISUAL_LATENCY, exit_wait_timed,
 			 wait);
 	gtk_dialog_run (GTK_DIALOG (wait));
 
@@ -359,7 +363,7 @@ grg_wait_message_change_reason (GtkWidget * wait, gchar * reason)
 	g_list_free (child1);
 	g_list_free (child2);
 
-	gtk_timeout_add (GRG_VISUAL_LATENCY, (GtkFunction) exit_wait_timed,
+	g_timeout_add (GRG_VISUAL_LATENCY, exit_wait_timed,
 			 wait);
 	gtk_dialog_run (GTK_DIALOG (wait));
 }
@@ -379,7 +383,7 @@ grg_display_file (gchar * file)
 		"xterm -e less %s &>/dev/null",
 		"/usr/X11R6/bin/xterm -e less %s &>/dev/null",
 		NULL
-	};			//add eterm, opera
+	};			/*add eterm, opera */
 
 	gchar *command = NULL;
 	gint resp = -1, i = 0;
@@ -390,7 +394,7 @@ grg_display_file (gchar * file)
 		return;
 	}
 
-	//iterates on the available visualizers list until it finds stng useable
+	/*iterates on the available visualizers list until it finds stng useable */
 	while (resp && commands[i])
 	{
 		gchar *quote = g_shell_quote (file);
@@ -401,3 +405,25 @@ grg_display_file (gchar * file)
 		i++;
 	}
 }
+
+GtkWidget * 
+grg_toolbar_insert_stock(GtkToolbar *toolbar,
+    const gchar *stock_id,
+    const char *tooltip_text,
+    GtkSignalFunc callback,
+    gpointer user_data,
+    gint position)
+{
+    GtkToolItem * item;
+
+    item = gtk_tool_button_new_from_stock(stock_id);
+
+    gtk_tool_item_set_tooltip_text (item, tooltip_text);
+
+    g_signal_connect (item, "clicked",
+			          callback, user_data);
+
+    gtk_toolbar_insert (toolbar, item, position);
+
+    return GTK_WIDGET (item);
+}
