@@ -27,6 +27,7 @@
 #include <locale.h>
 #include <time.h>
 #include <fcntl.h>
+#include <stdio.h>
 
 #include <gtk/gtk.h>
 
@@ -47,43 +48,47 @@
 
 #include <libgringotts.h>
 
-//appends a stock item to a toolbar
+#include "gringotts.h"
+
+/* appends a stock item to a toolbar */
 #define	TOOLBAR_INS_STOCK(tbar, stock, callback, tooltip) \
-	gtk_toolbar_insert_stock (GTK_TOOLBAR (tbar), stock, tooltip, \
-		"", (GtkSignalFunc) callback, NULL, -1)
+	grg_toolbar_insert_stock (GTK_TOOLBAR (tbar), stock, tooltip, \
+		(GtkSignalFunc) callback, NULL, -1)
 
-//appends a stock item to a toolbar, assigning it to a widget
+/* appends a stock item to a toolbar, assigning it to a widget */
 #define	TOOLBAR_INS_STOCK_WIDGET(tbar, stock, callback, tooltip, wid) \
-	wid = gtk_toolbar_insert_stock (GTK_TOOLBAR (tbar), stock, tooltip, \
-		"", (GtkSignalFunc) callback, NULL, -1)
+	wid = grg_toolbar_insert_stock (GTK_TOOLBAR (tbar), stock, tooltip, \
+		(GtkSignalFunc) callback, NULL, -1)
 
-//appends a stock item to a toolbar, assigning it to a widget and passing a value to the callback
+/* appends a stock item to a toolbar, assigning it to a widget and passing a value to the callback */
 #define	TOOLBAR_INS_STOCK_WIDGET_SIGNAL(tbar, stock, callback, tooltip, signal, wid) \
-	wid = gtk_toolbar_insert_stock (GTK_TOOLBAR (tbar), stock, tooltip, \
-		"", (GtkSignalFunc) callback, GINT_TO_POINTER (signal), -1)
+	wid = grg_toolbar_insert_stock (GTK_TOOLBAR (tbar), stock, tooltip, \
+		(GtkSignalFunc) callback, GINT_TO_POINTER (signal), -1)
 
-//appends a space to a toolbar
+/* appends a space to a toolbar */
 #define TOOLBAR_INS_SPACE(tbar) \
-	gtk_toolbar_append_space (GTK_TOOLBAR (tbar))
+	my_toolbar_append_space (GTK_TOOLBAR (tbar))
 
 /* - some menu buttons are never deactivated (i.e. Quit), so their widgets aren't really needed
  */
 
-// general
+/*  general */
 static GtkWidget *title, *win1, *edit, *lbl;
 
-// toolbar Navigation
+/*  toolbar Navigation */
 static GtkWidget *bfirst, *bback, *bfor, *blast, *bind;
 
-// others
+/*  others */
 static GtkWidget *btitle;
 
-// main toolbar 
+/*  main toolbar  */
 static GtkWidget *tnew, *topen, *tsave, *tclose;
 static GtkWidget *tadd, *trem, *tcut, *tcopy, *tpast, *tfind, *tpref;
-static GtkWidget *batadd, *batrem, *batsav, *batinf, *batchco, *combo_attach;
+static GtkWidget *batadd, *batrem, *batsav, *batinf, *batchco;
+static GtkComboBox *combo_attach;
+static GtkListStore * combo_attach_list_store;
 
-static guchar *grgfile = NULL, *caption = NULL;
+static gchar *grgfile = NULL, *caption = NULL;
 static gboolean started = FALSE, gtk_loop_started = FALSE;
 static gboolean created = FALSE;
 static guint tout;
@@ -94,6 +99,16 @@ GList *garbage = NULL;
 GRG_CTX gctx = NULL;
 glong pwdbirth = 0;
 
+static void
+my_toolbar_append_space (GtkToolbar * toolbar)
+{
+    GtkToolItem * separator;
+
+    separator = gtk_separator_tool_item_new();
+
+    gtk_toolbar_insert (toolbar, separator, -1);
+}
+
 /*
  * nyi:
  *
@@ -264,7 +279,7 @@ backup_file (gchar * filename)
 	gint res;
 
 	res = lstat (filename, &s);
-	if ((res < 0) || !S_ISREG (s.st_mode))	//file non-existent or non-regular
+	if ((res < 0) || !S_ISREG (s.st_mode))	/*file non-existent or non-regular */
 		return TRUE;
 
 	bak_name = g_strconcat (filename, BACKUP_SUFFIX, NULL);
@@ -298,16 +313,9 @@ meta_saveable (gpointer data, gpointer user_data)
 static void
 update_combo_attach (void)
 {
-	GtkWidget *old =
-		gtk_option_menu_get_menu (GTK_OPTION_MENU (combo_attach));
-	GtkWidget *new = grg_attachment_get_menu ();
-	gtk_widget_destroy (old);
-	gtk_option_menu_remove_menu (GTK_OPTION_MENU (combo_attach));
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (combo_attach), new);
+    grg_attachment_fill_combo_box (combo_attach);
 }
 
-void set_editor_font (const guchar * font_desc);
-
 /**
  * update:
  *
@@ -325,7 +333,7 @@ update (void)
 	gboolean moreThan1 = grg_entries_n_el () > 1;
 
 	update_combo_attach ();
-	//current_attach_ID gets aligned only after grg_attach_get_menu
+	/*current_attach_ID gets aligned only after grg_attach_get_menu */
 	isAttachSelected = current_attach_ID > -1;
 
 	gtk_widget_set_sensitive (btitle, isStuffed);
@@ -354,7 +362,7 @@ update (void)
 	gtk_widget_set_sensitive (batsav, isAttachSelected);
 	gtk_widget_set_sensitive (batinf, isAttachSelected);
 	gtk_widget_set_sensitive (batchco, isAttachSelected);
-	gtk_widget_set_sensitive (combo_attach, isAttachSelected);
+	gtk_widget_set_sensitive (GTK_WIDGET (combo_attach), isAttachSelected);
 
 	gtk_label_set_text (GTK_LABEL (title),
 			    isStuffed ? grg_entries_get_ID () : GRG_CAP_NAME
@@ -445,6 +453,9 @@ meta_quit (void)
 	if (file_close () == GRG_CANCEL)
 		return;
 
+	/* TODO: see mainwin_resize() */
+	grg_save_prefs();
+
 	quit (0);
 }
 
@@ -462,7 +473,7 @@ revert (void)
 	      _("You'll lose all the changes from\nlast save! Are you sure?"),
 	      FALSE, win1) == GRG_YES))
 	{
-		guchar *tmp = NULL;
+		gchar *tmp = NULL;
 		GtkWidget *wait = grg_wait_msg (_("loading"), win1);
 
 		gint err, fd;
@@ -586,31 +597,32 @@ revert (void)
  * Loads a gringotts file.
  */
 static void
-load_file (gchar * filename)
+load_file (gchar * input_filename)
 {
 	GtkWidget *wait;
 	GRG_KEY tmpkey;
 	gint err, fd;
-	guchar *res;
+	gchar *res;
 	struct stat buf1, buf2;
+    gchar * abs_filename = NULL;
 
-	if (!filename || !*filename)
-		return;
+	if (!input_filename || !*input_filename)
+		goto cleanup;
 
-	fd = grg_safe_open (filename);
+	fd = grg_safe_open (input_filename);
 
 	if (fd == GRG_OPEN_FILE_IRREGULAR)
 	{
 		grg_msg (_("You've selected a directory or a symlink"),
 			 GTK_MESSAGE_ERROR, win1);
-		return;
+		goto cleanup;
 	}
 
 	if (fd < 3)
 	{
 		grg_msg (_("The selected file doesn't exists"),
 			 GTK_MESSAGE_ERROR, win1);
-		return;
+		goto cleanup;
 	}
 
 	/* if this and the opened one are the very same file, fall back on revert() */
@@ -622,21 +634,22 @@ load_file (gchar * filename)
 	{
 		close (fd);
 		revert ();
-		return;
+		goto cleanup;
 	}
 
-	if (!g_path_is_absolute (filename))
+	if (g_path_is_absolute (input_filename))
+    {
+        abs_filename = g_strdup(input_filename);
+    }
+    else
 	{
-		gchar *tmpfname = (gchar *) grg_malloc (PATH_MAX);
+		abs_filename = (gchar *) grg_malloc (PATH_MAX);
 
-		realpath (filename, tmpfname);
-		g_free (filename);
-		filename = tmpfname;
-		tmpfname = NULL;
+		realpath (input_filename, abs_filename);
 	}
 
 	if (file_close () == GRG_CANCEL)
-		return;
+		goto cleanup;
 
 	err = grg_validate_file_direct (gctx, fd);
 
@@ -652,7 +665,7 @@ load_file (gchar * filename)
 		grg_msg (_
 			 ("This file doesn't seem to be a valid Gringotts one!"),
 			 GTK_MESSAGE_ERROR, win1);
-		return;
+		goto cleanup;
 	}
 
 	case GRG_MEM_ALLOCATION_ERR:
@@ -670,7 +683,7 @@ load_file (gchar * filename)
 		grg_msg (_
 			 ("Gringotts internal error. Cannot finish operation."),
 			 GTK_MESSAGE_ERROR, win1);
-		return;
+		goto cleanup;
 	}
 
 	case GRG_READ_FILE_ERR:
@@ -678,7 +691,7 @@ load_file (gchar * filename)
 		close (fd);
 		grg_msg (_("Uh-oh! I can't read from the file!"),
 			 GTK_MESSAGE_ERROR, win1);
-		return;
+		goto cleanup;
 	}
 
 	case GRG_READ_CRC_ERR:
@@ -687,14 +700,14 @@ load_file (gchar * filename)
 		close (fd);
 		grg_msg (_("The file appears to be corrupted!"),
 			 GTK_MESSAGE_ERROR, win1);
-		return;
+		goto cleanup;
 	}
 #ifdef GRG_READ_TOO_BIG_ERR
 	case GRG_READ_TOO_BIG_ERR:
 	{
 		close (fd);
 		grg_msg (_("File is too big"), GTK_MESSAGE_ERROR, win1);
-		return;
+		goto cleanup;
 	}
 #endif
 	default:
@@ -705,7 +718,7 @@ load_file (gchar * filename)
 			grg_msg (_
 				 ("Gringotts internal error. Cannot finish operation."),
 				 GTK_MESSAGE_ERROR, win1);
-			return;
+			goto cleanup;
 		}
 	}
 	}
@@ -720,12 +733,12 @@ load_file (gchar * filename)
 		if (!tmpkey)
 		{
 			close (fd);
-			return;
+			goto cleanup;
 		}
 
 		wait = grg_wait_msg (_("loading"), win1);
 
-		err = grg_load_wrapper (&res, tmpkey, fd, filename);
+		err = grg_load_wrapper (&res, tmpkey, fd, abs_filename);
 
 		if (err < 0)
 			gtk_widget_destroy (wait);
@@ -799,7 +812,7 @@ load_file (gchar * filename)
 		{
 			close (fd);
 			grg_key_free (gctx, tmpkey);
-			return;
+			goto cleanup;
 		}
 
 		if (exit)
@@ -811,7 +824,7 @@ load_file (gchar * filename)
 	g_free (grgfile);
 	grgfile = NULL;
 
-	grgfile = g_strdup (filename);
+	grgfile = g_strdup (abs_filename);
 	grg_key_free (gctx, key);
 	key = grg_key_clone (tmpkey);
 	grg_key_free (gctx, tmpkey);
@@ -825,6 +838,13 @@ load_file (gchar * filename)
 			 ("The current password is expired.\nYou should change it, or modify this "
 			  "setting in the preferences"), GTK_MESSAGE_WARNING,
 			 win1);
+
+cleanup:
+    if (abs_filename)
+    {
+        g_free (abs_filename);
+    }
+    return;
 }
 
 /**
@@ -835,26 +855,36 @@ load_file (gchar * filename)
 void
 meta_load (void)
 {
-	GtkWidget *file_selector;
+    GtkWidget *file_chooser;
 	gint response;
-	file_selector = gtk_file_selection_new (_("Open..."));
-	gtk_window_set_transient_for (GTK_WINDOW (file_selector),
-				      GTK_WINDOW (win1));
-	gtk_widget_show (file_selector);
-	response = gtk_dialog_run (GTK_DIALOG (file_selector));
-	if (response == GTK_RESPONSE_OK)
-	{
-		//it may appear a stupid duplication, but it's very important
-		//to avoid two file selectors at the same time, so one should
-		//be destroyed *before* calling load_file
-		gchar *fname = strdup (gtk_file_selection_get_filename
-				       (GTK_FILE_SELECTION (file_selector)));
-		gtk_widget_destroy (file_selector);
-		load_file (fname);
-		g_free (fname);
-	}
-	else
-		gtk_widget_destroy (file_selector);
+
+    file_chooser = gtk_file_chooser_dialog_new (_("Open..."),
+            GTK_WINDOW(win1),
+            GTK_FILE_CHOOSER_ACTION_OPEN,
+            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+            GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+            NULL);
+
+	response = gtk_dialog_run (GTK_DIALOG (file_chooser));
+    if (response == GTK_RESPONSE_ACCEPT)
+    {
+		/*
+         * It may appear a stupid duplication, but it's very important
+         * to avoid two file selectors at the same time, so one should
+         * be destroyed *before* calling load_file
+         * */
+        char *fname;
+
+        fname = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_chooser));
+        gtk_widget_destroy (file_chooser);
+        load_file (fname);
+
+        g_free (fname);
+    }
+    else
+    {
+        gtk_widget_destroy (file_chooser);
+    }
 }
 
 /**
@@ -972,10 +1002,12 @@ retitle (void)
 static void
 save_as (const gchar * fpath)
 {
+#if 0
 	GtkWidget *wait;
+#endif
 	gchar *tmpfile;
 	gint err, fd;
-	gboolean is_current = STR_EQ (fpath, grgfile);	//Am I saving the current file?
+	gboolean is_current = STR_EQ (fpath, grgfile);	/*Am I saving the current file? */
 
 	tmpfile = g_strdup (fpath);
 
@@ -985,7 +1017,7 @@ save_as (const gchar * fpath)
 		    (fpath + strlen (fpath) - SUFFIX_LEN, SUFFIX,
 		     SUFFIX_LEN) != 0)
 		{
-			guchar *tmp = g_strconcat (tmpfile, SUFFIX, NULL);
+			gchar *tmp = g_strconcat (tmpfile, SUFFIX, NULL);
 			g_free (tmpfile);
 			tmpfile = g_strdup (tmp);
 			g_free (tmp);
@@ -1114,17 +1146,26 @@ save_as (const gchar * fpath)
 void
 meta_save_as (void)
 {
-	GtkWidget *file_selector;
-	gint response;
-	file_selector = gtk_file_selection_new (_("Save as...."));
-	gtk_window_set_transient_for (GTK_WINDOW (file_selector),
-				      GTK_WINDOW (win1));
-	gtk_widget_show (file_selector);
-	response = gtk_dialog_run (GTK_DIALOG (file_selector));
-	if (response == GTK_RESPONSE_OK)
-		save_as (gtk_file_selection_get_filename
-			 (GTK_FILE_SELECTION (file_selector)));
-	gtk_widget_destroy (file_selector);
+    GtkWidget *file_chooser;
+    gint response;
+    file_chooser = gtk_file_chooser_dialog_new (_("Save as...."),
+                                      GTK_WINDOW(win1),
+                                      GTK_FILE_CHOOSER_ACTION_SAVE,
+                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+                                      NULL);
+    gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (file_chooser), TRUE);
+    response = gtk_dialog_run (GTK_DIALOG (file_chooser));
+    if (response == GTK_RESPONSE_ACCEPT)
+    {
+        char * filename;
+
+        filename = gtk_file_chooser_get_filename
+            (GTK_FILE_CHOOSER (file_chooser));
+        save_as (filename);
+        g_free(filename);
+    }
+    gtk_widget_destroy (file_chooser);
 }
 
 /**
@@ -1198,12 +1239,15 @@ chpwd (void)
 	verkey = grg_ask_pwd_dialog (win1);
 
 	if (!verkey || !grg_key_compare (verkey, key))
-	{
-		grg_msg (_("Wrong password"), GTK_MESSAGE_ERROR, win1);
-		grg_key_free (gctx, verkey);
-		verkey = NULL;
-		return;
-	}
+    {
+        grg_msg (_("Wrong password"), GTK_MESSAGE_ERROR, win1);
+        if (verkey)
+        {
+            grg_key_free (gctx, verkey);
+            verkey = NULL;
+        }
+        return;
+    }
 
 	grg_key_free (gctx, verkey);
 	verkey = NULL;
@@ -1240,12 +1284,15 @@ static void
 destroy_splash (GtkWidget * w, GdkEvent * ev, GtkWidget * w2)
 {
 	gtk_widget_destroy (w2);
-	gtk_timeout_remove (tout);
+	g_source_remove (tout);
 }
 
 static gboolean
-destroy_splash_timed (GtkWidget * w)
+destroy_splash_timed (gpointer void_w)
 {
+    GtkWidget * w;
+
+    w = (GtkWidget *)void_w;
 	gtk_widget_destroy (w);
 	return FALSE;
 }
@@ -1282,8 +1329,8 @@ grg_splash (GtkWidget * parent)
 	g_signal_connect (G_OBJECT (spebox), "button-press-event",
 			  G_CALLBACK (destroy_splash), spwin);
 
-	tout = gtk_timeout_add (GRG_SPLASH_TIMEOUT,
-				(GtkFunction) destroy_splash_timed, spwin);
+	tout = g_timeout_add (GRG_SPLASH_TIMEOUT,
+				(GSourceFunc) destroy_splash_timed, spwin);
 
 	return spwin;
 }
@@ -1291,26 +1338,36 @@ grg_splash (GtkWidget * parent)
 static void
 attach_file (void)
 {
-	GtkWidget *file_selector;
+    GtkWidget *file_chooser;
 	gint response;
 	gchar *selection = NULL;
 
 	sync_entry ();
 
-	file_selector = gtk_file_selection_new (_("Select file..."));
-	gtk_window_set_transient_for (GTK_WINDOW (file_selector),
-				      GTK_WINDOW (win1));
-	gtk_widget_show (file_selector);
-	response = gtk_dialog_run (GTK_DIALOG (file_selector));
-	if (response == GTK_RESPONSE_OK)
-		selection = g_strdup (gtk_file_selection_get_filename
-				      (GTK_FILE_SELECTION (file_selector)));
-	gtk_widget_destroy (file_selector);
-	if (response != GTK_RESPONSE_OK)
+    file_chooser = gtk_file_chooser_dialog_new (_("Select file..."),
+            GTK_WINDOW(win1),
+            GTK_FILE_CHOOSER_ACTION_OPEN,
+            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+            GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+            NULL);
+
+    response = gtk_dialog_run (GTK_DIALOG (file_chooser));
+    if (response == GTK_RESPONSE_ACCEPT)
+    {
+		/*
+         * It may appear a stupid duplication, but it's very important
+         * to avoid two file selectors at the same time, so one should
+         * be destroyed *before* calling load_file
+         * */
+        selection = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_chooser));
+    }
+
+    gtk_widget_destroy (file_chooser);
+	if (response != GTK_RESPONSE_ACCEPT)
 		return;
 	response = grg_attach_file (selection, win1);
 	GRGAFREE (selection);
-	if (response < 0)	//didn't change anything
+	if (response < 0)	/*didn't change anything */
 		return;
 
 	update_saveable (GRG_SAVE_ACTIVE);
@@ -1337,21 +1394,28 @@ detach_file (void)
 static void
 save_attached_file (void)
 {
-	GtkWidget *file_selector;
-	gint response;
-	gchar *selection = NULL;
+    GtkWidget *file_chooser;
+    gint response;
+    gchar *selection = NULL;
 
-	file_selector = gtk_file_selection_new (_("Save as...."));
-	gtk_window_set_transient_for (GTK_WINDOW (file_selector),
-				      GTK_WINDOW (win1));
-	gtk_widget_show (file_selector);
-	response = gtk_dialog_run (GTK_DIALOG (file_selector));
-	if (response == GTK_RESPONSE_OK)
-		selection = g_strdup (gtk_file_selection_get_filename
-				      (GTK_FILE_SELECTION (file_selector)));
-	gtk_widget_destroy (file_selector);
-	if (response != GTK_RESPONSE_OK)
+    file_chooser = gtk_file_chooser_dialog_new (_("Save as...."),
+                                      GTK_WINDOW(win1),
+                                      GTK_FILE_CHOOSER_ACTION_SAVE,
+                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+                                      NULL);
+
+    response = gtk_dialog_run (GTK_DIALOG (file_chooser));
+	if (response == GTK_RESPONSE_ACCEPT)
+    {
+		selection = gtk_file_chooser_get_filename
+            (GTK_FILE_CHOOSER (file_chooser));
+    }
+	gtk_widget_destroy (file_chooser);
+	if (response != GTK_RESPONSE_ACCEPT)
+    {
 		return;
+    }
 	grg_save_attachment (selection, win1);
 	GRGAFREE (selection);
 }
@@ -1359,21 +1423,31 @@ save_attached_file (void)
 void
 wipe_file (void)
 {
-	GtkWidget *file_selector, *wait;
+    GtkWidget *file_chooser, *wait;
 	gint response;
-	gchar *selection = NULL;
+    gchar *selection = NULL;
 
-	file_selector = gtk_file_selection_new (_("File to wipe"));
-	gtk_window_set_transient_for (GTK_WINDOW (file_selector),
-				      GTK_WINDOW (win1));
-	gtk_widget_show (file_selector);
-	response = gtk_dialog_run (GTK_DIALOG (file_selector));
-	if (response == GTK_RESPONSE_OK)
-		selection = g_strdup (gtk_file_selection_get_filename
-				      (GTK_FILE_SELECTION (file_selector)));
-	gtk_widget_destroy (file_selector);
-	if (response != GTK_RESPONSE_OK)
+    file_chooser = gtk_file_chooser_dialog_new (_("File to wipe"),
+            GTK_WINDOW(win1),
+            GTK_FILE_CHOOSER_ACTION_OPEN,
+            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+            GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+            NULL);
+
+	response = gtk_dialog_run (GTK_DIALOG (file_chooser));
+    if (response == GTK_RESPONSE_ACCEPT)
+    {
+		selection = gtk_file_chooser_get_filename (
+                GTK_FILE_CHOOSER (file_chooser)
+                );
+    }
+
+    gtk_widget_destroy (file_chooser);
+
+	if (response != GTK_RESPONSE_ACCEPT)
+    {
 		return;
+    }
 
 	if (!g_file_test (selection, G_FILE_TEST_IS_REGULAR))
 	{
@@ -1418,7 +1492,7 @@ change_attach_comment (void)
 		update_saveable (GRG_SAVE_ACTIVE);
 }
 
-guchar *
+gchar *
 get_editor_font (void)
 {
 	PangoContext *editorFont = gtk_widget_get_pango_context (edit);
@@ -1428,7 +1502,7 @@ get_editor_font (void)
 }
 
 void
-set_editor_font (const guchar * font_desc)
+set_editor_font (const gchar * font_desc)
 {
 	PangoFontDescription *fdesc =
 		pango_font_description_from_string (font_desc);
@@ -1436,6 +1510,19 @@ set_editor_font (const guchar * font_desc)
 	pango_font_description_free (fdesc);
 }
 
+static void
+mainwin_resize (GtkWidget *widget, GtkAllocation *allocation)
+{
+	g_return_if_fail(allocation != NULL);
+
+	grg_prefs_mainwin_width = allocation->width;
+	grg_prefs_mainwin_height = allocation->height;
+	/* TODO: in theory, prefs saveable status should be updated with GRG_SET_ACTIVE here,
+	but because current prefs design doesn't make the difference between GUI-prefs
+	dirty status and open-file dirty status, grg_save_prefs() is unconditionally
+	called when leaving */
+}
+
 /**
  * grg_interface:
  *
@@ -1450,8 +1537,10 @@ grg_interface (void)
 	GtkSizeGroup *resizer;
 	gchar *str, *fdesc;
 	PangoFontDescription *pfd;
+    GtkCellRenderer *cell;
+	GdkGeometry geometry;
 
-	// window
+	/* window */
 	win1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	grg_window_set_icon (GTK_WINDOW (win1));
 
@@ -1465,7 +1554,7 @@ grg_interface (void)
 	g_signal_connect (G_OBJECT (win1), "destroy", G_CALLBACK (meta_quit),
 			  NULL);
 
-	// the multi-line text widget
+	/* the multi-line text widget */
 	scroll = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scroll),
 					     GTK_SHADOW_ETCHED_IN);
@@ -1473,16 +1562,14 @@ grg_interface (void)
 					GTK_POLICY_AUTOMATIC,
 					GTK_POLICY_AUTOMATIC);
 	
-	gtk_widget_set_size_request (scroll, 400, 300);
-	
 	edit = get_updated_sheet (FALSE);
 	
 	gtk_container_add (GTK_CONTAINER (scroll), edit);
 
-	// the title widget
+	/* the title widget */
 	title = gtk_label_new ("");
 	gtk_misc_set_alignment (GTK_MISC (title), 0.1, 0.5);
-	btitle = gtk_button_new_with_mnemonic (_("_Edit..."));
+	btitle = gtk_button_new_with_mnemonic (_("E_dit..."));
 	g_signal_connect (G_OBJECT (btitle), "clicked", G_CALLBACK (retitle),
 			  NULL);
 	pfd = pango_font_description_new ();
@@ -1491,13 +1578,11 @@ grg_interface (void)
 	pango_font_description_free (pfd);
 	lbl = gtk_label_new ("");
 
-	// the "navigation" lateral toolbar
+	/* the "navigation" lateral toolbar */
 	tbar_nav = gtk_toolbar_new ();
 	gtk_toolbar_set_style (GTK_TOOLBAR (tbar_nav), GTK_TOOLBAR_ICONS);
 	gtk_toolbar_set_orientation (GTK_TOOLBAR (tbar_nav),
 				     GTK_ORIENTATION_VERTICAL);
-	gtk_toolbar_set_icon_size (GTK_TOOLBAR (tbar_nav),
-				   GTK_ICON_SIZE_LARGE_TOOLBAR);
 	handle_nav = gtk_handle_box_new ();
 	gtk_handle_box_set_handle_position (GTK_HANDLE_BOX (handle_nav),
 					    GTK_POS_TOP);
@@ -1525,23 +1610,30 @@ grg_interface (void)
 					 move_around, _("Go to first entry"),
 					 GRG_MV_FIRST, bfirst);
 
-	// size group for "left column"
+	/* size group for "left column" */
 	resizer = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 	gtk_size_group_add_widget (resizer, btitle);
 	gtk_size_group_add_widget (resizer, tbar_nav);
 
-	//the "main" toolbar
+	/*the "main" toolbar */
 	tbar_main = gtk_toolbar_new ();
-	gtk_toolbar_set_icon_size (GTK_TOOLBAR (tbar_main),
-				   GTK_ICON_SIZE_LARGE_TOOLBAR);
 	gtk_toolbar_set_style (GTK_TOOLBAR (tbar_main), GTK_TOOLBAR_ICONS);
 	handle_main = gtk_handle_box_new ();
 	gtk_container_add (GTK_CONTAINER (handle_main), tbar_main);
 
 	str = grg_get_security_text (_("Security level: %s"));
-	gtk_toolbar_append_item (GTK_TOOLBAR (tbar_main), _("Security"), str,
-				 NULL, grg_get_security_button (),
-				 grg_security_monitor, NULL);
+
+    {
+        GtkToolItem * button;
+        button = gtk_tool_button_new (grg_get_security_button(), _("Security"));
+
+        gtk_tool_item_set_tooltip_text (button, str);
+        
+        g_signal_connect (button, "clicked",
+            grg_security_monitor, NULL);
+        
+        gtk_toolbar_insert (GTK_TOOLBAR (tbar_main), button, -1);
+    }
 	g_free (str);
 
 	TOOLBAR_INS_SPACE (tbar_main);
@@ -1591,10 +1683,8 @@ grg_interface (void)
 	TOOLBAR_INS_STOCK (tbar_main, GTK_STOCK_QUIT, meta_quit,
 			   _("Quit Gringotts"));
 
-	//attachment handling toolbar
+	/*attachment handling toolbar */
 	tbar_attach = gtk_toolbar_new ();
-	gtk_toolbar_set_icon_size (GTK_TOOLBAR (tbar_attach),
-				   GTK_ICON_SIZE_LARGE_TOOLBAR);
 	gtk_toolbar_set_style (GTK_TOOLBAR (tbar_attach), GTK_TOOLBAR_ICONS);
 	handle_attach = gtk_handle_box_new ();
 	gtk_container_add (GTK_CONTAINER (handle_attach), tbar_attach);
@@ -1603,9 +1693,17 @@ grg_interface (void)
 			    1);
 	gtk_box_pack_start (GTK_BOX (scrollbox), scroll, TRUE, TRUE, 1);
 
-	gtk_toolbar_append_widget (GTK_TOOLBAR (tbar_attach),
-				   gtk_label_new (_("Attached files")), "",
-				   "");
+    {
+        GtkToolItem * button;
+
+        button = gtk_tool_item_new ();
+        gtk_container_add (GTK_CONTAINER (button), 
+                gtk_label_new (_("Attached files")));
+        
+        gtk_tool_item_set_tooltip_text (button, "");
+
+        gtk_toolbar_insert (GTK_TOOLBAR (tbar_attach), button, -1);
+    }
 
 /*	TOOLBAR_INS_STOCK (tbar_attach, GTK_STOCK_DIALOG_WARNING, attach_warn,
 			   _("Important informations\non this feature"));*/
@@ -1623,11 +1721,31 @@ grg_interface (void)
 				  change_attach_comment, _("Change comment"),
 				  batchco);
 
-	combo_attach = gtk_option_menu_new ();
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (combo_attach),
-				  grg_attachment_get_menu ());
-	gtk_toolbar_append_widget (GTK_TOOLBAR (tbar_attach), combo_attach,
-				   _("List of attached files"), "");
+    combo_attach_list_store =
+        gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_INT);
+    combo_attach =
+        GTK_COMBO_BOX (gtk_combo_box_new_with_model (
+                GTK_TREE_MODEL (combo_attach_list_store)
+                ));
+
+    cell = gtk_cell_renderer_text_new ();
+    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo_attach), cell, TRUE);
+    gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo_attach), cell,
+                                    "text", ATTACHMENT_TITLE,
+                                    NULL);
+
+    grg_attachment_fill_combo_box (combo_attach);
+    {
+        GtkToolItem * button;
+
+        button = gtk_tool_item_new ();
+        gtk_container_add (GTK_CONTAINER (button), 
+                GTK_WIDGET (combo_attach));
+        
+        gtk_tool_item_set_tooltip_text (button, _("List of attached files"));
+
+        gtk_toolbar_insert (GTK_TOOLBAR (tbar_attach), button, -1);
+    }
 
 	vbox = gtk_vbox_new (FALSE, 1);
 	gtk_widget_show (vbox);
@@ -1666,8 +1784,21 @@ grg_interface (void)
 
 	update ();
 
+	/* set minimum size hints to the main window */
+	geometry.min_width = 460;
+	geometry.min_height = 460;
+	gtk_window_set_geometry_hints (GTK_WINDOW (win1), NULL,
+									&geometry, GDK_HINT_MIN_SIZE);
+	/* and apply custom size preferences */
+	gtk_widget_set_size_request (win1, grg_prefs_mainwin_width,	
+									grg_prefs_mainwin_height);
+	g_signal_connect (G_OBJECT (win1), "size_allocate", G_CALLBACK (mainwin_resize),
+			  NULL);
+
 	started = TRUE;
 	gtk_widget_show_all (win1);
+
+
 	if (grg_prefs_splash)
 		gtk_widget_show_all (grg_splash (win1));
 }
@@ -1685,16 +1816,16 @@ gint
 main (gint argc, gchar ** argv)
 {
 	gchar *file2load = NULL, *file2loadInArgv = NULL, *finalfile = NULL;
-	gchar *version = grg_get_version (); //libgringotts version
+	guchar *version = grg_get_version (); /* libgringotts version */
 	gint prefs_err;
 	gboolean root = FALSE;
 
 	if (!grg_mlockall_and_drop_root_privileges ())
 		exit (1);
 
-	gctx = grg_context_initialize_defaults ("GRG");
+	gctx = grg_context_initialize_defaults ((guchar*)"GRG");
 
-	//parse cmdline args
+	/*parse cmdline args */
 	grg_parse_argv (argc, argv, &file2loadInArgv, &root);
 
 	if (!grg_security_filter (root))
@@ -1726,7 +1857,7 @@ main (gint argc, gchar ** argv)
 
 	grg_interface ();
 
-	//if the preferences file is invalid, saves a default
+	/*if the preferences file is invalid, saves a default */
 	if (prefs_err < 0)
 	{
 		g_warning ("%s",
@@ -1736,7 +1867,7 @@ main (gint argc, gchar ** argv)
 		grg_save_prefs ();
 	}
 
-	//loads (ev.) a startup file
+	/*loads (ev.) a startup file */
 	file2load = get_pref_file ();
 
 	if (file2loadInArgv)
